@@ -65,6 +65,27 @@ ml_pty_new(
 	}
 }
 
+ml_pty_ptr_t
+ml_pty_new_with(
+	int  master ,
+	int  slave ,
+	pid_t  child_pid ,
+	u_int  cols ,
+	u_int  rows
+	)
+{
+#ifndef  USE_WIN32API
+	if( ptsname( master))
+	{
+		return  ml_pty_unix_new_with( master , slave , child_pid , cols , rows) ;
+	}
+	else
+#endif
+	{
+		return  NULL ;
+	}
+}
+
 int
 ml_pty_delete(
 	ml_pty_t *  pty
@@ -74,6 +95,13 @@ ml_pty_delete(
 	{
 		(*pty->pty_listener->closed)( pty->pty_listener->self) ;
 	}
+#ifdef  DEBUG
+	else
+	{
+		kik_debug_printf( KIK_DEBUG_TAG " %s is not set.\n" ,
+			pty->pty_listener ? "pty_listener->closed" : "pty listener") ;
+	}
+#endif
 
 	free( pty->buf) ;
 
