@@ -146,14 +146,23 @@ ml_term_zombie(
 	ml_term_t *  term
 	)
 {
-	ml_pty_ptr_t  pty ;
+	if( term->pty)
+	{
+		ml_pty_ptr_t  pty ;
 
-	pty = term->pty ;
+		pty = term->pty ;
 
-	/* Should be NULL because ml_pty_delete calls term->pty_listener->closed. */
-	term->pty = NULL ;
+		/* Should be NULL because ml_pty_delete calls term->pty_listener->closed. */
+		term->pty = NULL ;
 
-	ml_pty_delete( pty) ;
+		ml_pty_delete( pty) ;
+	}
+#ifdef  DEBUG
+	else
+	{
+		kik_debug_printf( KIK_DEBUG_TAG " term is already zombie.\n") ;
+	}
+#endif
 
 	return  1 ;
 }
@@ -170,18 +179,13 @@ ml_term_open_pty(
 	const char *  privkey
 	)
 {
-	if( term->pty)
+	if( ! term->pty)
 	{
-		/* already opened */
-		
-		return  1 ;
-	}
-	else
-	{
-		if( ( term->pty = ml_pty_new( cmd_path , argv , env , host , pass ,
-					pubkey , privkey ,
-					ml_screen_get_logical_cols( term->screen) ,
-					ml_screen_get_logical_rows( term->screen))) == NULL)
+		ml_pty_ptr_t  pty ;
+
+		if( ! ( pty = ml_pty_new( cmd_path , argv , env , host , pass , pubkey , privkey ,
+				ml_screen_get_logical_cols( term->screen) ,
+				ml_screen_get_logical_rows( term->screen))))
 		{
 		#ifdef  DEBUG
 			kik_warn_printf( KIK_DEBUG_TAG " ml_pty_new failed.\n") ;
@@ -190,6 +194,22 @@ ml_term_open_pty(
 			return  0 ;
 		}
 
+		ml_term_plug_pty( term , pty) ;
+	}
+
+	return  1 ;
+}
+
+int
+ml_term_plug_pty(
+	ml_term_t *  term ,
+	ml_pty_ptr_t  pty	/* Not NULL */
+	)
+{
+	if( ! term->pty)
+	{
+		term->pty = pty ;
+
 		if( term->pty_listener)
 		{
 			ml_pty_set_listener( term->pty, term->pty_listener) ;
@@ -197,9 +217,9 @@ ml_term_open_pty(
 		}
 
 		ml_vt100_parser_set_pty( term->parser , term->pty) ;
-
-		return  1 ;
 	}
+	
+	return  1 ;
 }
 
 int
@@ -331,7 +351,7 @@ ml_term_set_use_dynamic_comb(
 }
 
 int
-ml_term_get_pty_fd(
+ml_term_get_master_fd(
 	ml_term_t *  term
 	)
 {
@@ -343,6 +363,19 @@ ml_term_get_pty_fd(
 	return  ml_pty_get_master_fd( term->pty) ;
 }
 
+int
+ml_term_get_slave_fd(
+	ml_term_t *  term
+	)
+{
+	if( term->pty == NULL)
+	{
+		return  -1 ;
+	}
+	
+	return  ml_pty_get_slave_fd( term->pty) ;
+}
+
 /*
  * Always return non-NULL value.
  * XXX Static data can be returned. (Not reentrant)
