@@ -140,7 +140,7 @@ x_prepare_for_main_config(
 	kik_conf_add_opt( conf , 'V' , "varwidth" , 1 , "use_variable_column_width" ,
 		"variable column width (for proportional/ISCII) [false]") ;
 	kik_conf_add_opt( conf , 'X' , "alpha" , 0 , "alpha" ,
-		"alpha blending for translucent [210]") ;
+		"alpha blending for translucent [255]") ;
 	kik_conf_add_opt( conf , 'Z' , "multicol" , 1 , "use_multi_column_char" ,
 		"fullwidth character occupies two logical columns [true]") ;
 	kik_conf_add_opt( conf , 'a' , "ac" , 0 , "col_size_of_width_a" ,
@@ -229,6 +229,8 @@ x_prepare_for_main_config(
 		"use unicode property for characters [false]") ;
 	kik_conf_add_opt( conf , '\0' , "logmsg" , 1 , "logging_msg" ,
 		"output messages to ~/.mlterm/msg[pid].log [true]") ;
+	kik_conf_add_opt( conf , '\0' , "osc52" , 1 , "allow_osc52" ,
+		"allow access to clipboard by OSC 52 sequence [false]") ;
 	kik_conf_set_end_opt( conf , 'e' , NULL , "exec_cmd" , 
 		"execute external command") ;
 
@@ -1174,6 +1176,16 @@ x_main_config_init(
 	}
 #endif
 
+	main_config->allow_osc52 = 0 ;
+
+	if( ( value = kik_conf_get_value( conf , "allow_osc52")))
+	{
+		if( strcmp( value , "true") == 0)
+		{
+			main_config->allow_osc52 = 1 ;
+		}
+	}
+
 	if( ( value = kik_conf_get_value( conf , "exec_cmd")) && strcmp( value , "true") == 0)
 	{
 		if( ( main_config->cmd_argv = malloc( sizeof( char*) * (argc + 1))) == NULL)
