@@ -70,8 +70,6 @@ static u_int  num_of_startup_ptys ;
 
 static x_system_event_listener_t  system_listener ;
 
-static char *  version ;
-
 static x_main_config_t  main_config ;
 
 static x_color_config_t  color_config ;
@@ -121,24 +119,6 @@ sig_fatal( int  sig)
  * Callbacks of ml_config_event_listener_t events.
  */
  
-static kik_conf_t *
-conf_new(void)
-{
-	kik_conf_t *  conf ;
-	
-	if( ( conf = kik_conf_new( "mlterm" ,
-		MAJOR_VERSION , MINOR_VERSION , REVISION , PATCH_LEVEL , CHANGE_DATE)) == NULL)
-	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " kik_conf_new() failed.\n") ;
-	#endif
-	
-		return  NULL ;
-	}
-
-	return  conf ;
-}
-
 /*
  * Reload mlterm/main file and reset main_config.
  * Notice: Saved changes are not applied to the screens already opened.
@@ -151,7 +131,7 @@ config_saved(void)
 
 	x_main_config_final( &main_config) ;
 
-	if( ( conf = conf_new()) == NULL)
+	if( ( conf = kik_conf_new()) == NULL)
 	{
 		return ;
 	}
@@ -289,21 +269,15 @@ open_pty_intern(
 	char *  env[6] ;	/* MLTERM,TERM,WINDOWID,DISPLAY,COLORFGBG,NULL */
 	char **  env_p ;
 	char  wid_env[9 + DIGIT_STR_LEN(Window) + 1] ;	/* "WINDOWID="(9) + [32bit digit] + NULL(1) */
-	char *  ver_env ;
 	char *  disp_env ;
 	char *  term_env ;
-	char *  colorfgbg_env ;
 	char *  uri ;
 	char *  pass ;
 	int  ret ;
 
 	env_p = env ;
 
-	if( version && ( ver_env = alloca( 7 + strlen( version) + 1)))
-	{
-		sprintf( ver_env , "MLTERM=%s" , version) ;
-		*(env_p ++) = ver_env ;
-	}
+	*(env_p ++) = "MLTERM=" VERSION ;
 	
 	sprintf( wid_env , "WINDOWID=%ld" , window) ;
 	*(env_p ++) = wid_env ;
@@ -323,11 +297,7 @@ open_pty_intern(
 		*(env_p ++) = term_env ;
 	}
 
-	/* COLORFGBG=default;default */
-	if( ( colorfgbg_env = kik_str_alloca_dup( "COLORFGBG=default;default")))
-	{
-		*(env_p ++) = colorfgbg_env ;
-	}
+	*(env_p ++) = "COLORFGBG=default;default" ;
 
 	/* NULL terminator */
 	*env_p = NULL ;
@@ -661,7 +631,7 @@ open_screen_intern(
 			main_config.conf_menu_path_2 , main_config.conf_menu_path_3 ,
 			main_config.use_extended_scroll_shortcut ,
 			main_config.borderless , main_config.line_space ,
-			main_config.input_method)) == NULL)
+			main_config.input_method , main_config.allow_osc52)) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " x_screen_new() failed.\n") ;
@@ -832,7 +802,7 @@ __exit(
 {
 #ifdef  KIK_DEBUG
 #ifdef  USE_WIN32GUI
-	int  count ;
+	u_int  count ;
 
 	for( count = 0 ; count < num_of_screens ; count++)
 	{
@@ -868,45 +838,35 @@ open_pty(
 	char *  dev
 	)
 {
-	int  count ;
+	ml_term_t *  new ;
 
-	for( count = 0 ; count < num_of_screens ; count ++)
+	if( dev)
 	{
-		if( screen == screens[count])
+		if( ( new = ml_get_detached_term( dev)) == NULL)
 		{
-			ml_term_t *  new ;
-
-			if( dev)
-			{
-				if( ( new = ml_get_detached_term( dev)) == NULL)
-				{
-					return ;
-				}
-			}
-			else
-			{
-				if( ( new = create_term_intern()) == NULL)
-				{
-					return ;
-				}
-
-				if( ! open_pty_intern( new , main_config.cmd_path ,
-					main_config.cmd_argv ,
-					DisplayString(  screen->window.disp->display) ,
-					x_get_root_window( &screen->window)->my_window))
-				{
-					ml_destroy_term( new) ;
-					
-					return ;
-				}
-			}
-			
-			x_screen_detach( screen) ;
-			x_screen_attach( screen , new) ;
-			
 			return ;
 		}
 	}
+	else
+	{
+		if( ( new = create_term_intern()) == NULL)
+		{
+			return ;
+		}
+
+		if( ! open_pty_intern( new , main_config.cmd_path ,
+			main_config.cmd_argv ,
+			DisplayString(  screen->window.disp->display) ,
+			x_get_root_window( &screen->window)->my_window))
+		{
+			ml_destroy_term( new) ;
+
+			return ;
+		}
+	}
+
+	x_screen_detach( screen) ;
+	x_screen_attach( screen , new) ;
 }
 
 static void
@@ -915,31 +875,21 @@ next_pty(
 	x_screen_t *  screen
 	)
 {
-	int  count ;
-	
-	for( count = 0 ; count < num_of_screens ; count ++)
+	ml_term_t *  old ;
+	ml_term_t *  new ;
+
+	if( ( old = x_screen_detach( screen)) == NULL)
 	{
-		if( screen == screens[count])
-		{
-			ml_term_t *  old ;
-			ml_term_t *  new ;
+		return ;
+	}
 
-			if( ( old = x_screen_detach( screen)) == NULL)
-			{
-				return ;
-			}
-
-			if( ( new = ml_next_term( old)) == NULL)
-			{
-				x_screen_attach( screen , old) ;
-			}
-			else
-			{
-				x_screen_attach( screen , new) ;
-			}
-			
-			return ;
-		}
+	if( ( new = ml_next_term( old)) == NULL)
+	{
+		x_screen_attach( screen , old) ;
+	}
+	else
+	{
+		x_screen_attach( screen , new) ;
 	}
 }
 
@@ -949,31 +899,21 @@ prev_pty(
 	x_screen_t *  screen
 	)
 {
-	int  count ;
-	
-	for( count = 0 ; count < num_of_screens ; count ++)
-	{
-		if( screen == screens[count])
-		{
-			ml_term_t *  old ;
-			ml_term_t *  new ;
+	ml_term_t *  old ;
+	ml_term_t *  new ;
 
-			if( ( old = x_screen_detach( screen)) == NULL)
-			{
-				return ;
-			}
-			
-			if( ( new = ml_prev_term( old)) == NULL)
-			{
-				x_screen_attach( screen , old) ;
-			}
-			else
-			{
-				x_screen_attach( screen , new) ;
-			}
-			
-			return ;
-		}
+	if( ( old = x_screen_detach( screen)) == NULL)
+	{
+		return ;
+	}
+
+	if( ( new = ml_prev_term( old)) == NULL)
+	{
+		x_screen_attach( screen , old) ;
+	}
+	else
+	{
+		x_screen_attach( screen , new) ;
 	}
 }
 
@@ -1060,7 +1000,7 @@ close_screen(
 	x_screen_t *  screen
 	)
 {
-	int  count ;
+	u_int  count ;
 	
 	for( count = 0 ; count < num_of_screens ; count ++)
 	{
@@ -1186,7 +1126,7 @@ mlclient(
 			pty = NULL ;
 		}
 
-		if( ( conf = conf_new()) == NULL)
+		if( ( conf = kik_conf_new()) == NULL)
 		{
 			return  0 ;
 		}
@@ -1537,7 +1477,7 @@ receive_next_event(void)
 
 		for( count = 0 ; count < num_of_terms ; count ++)
 		{
-			ptyfd = ml_term_get_pty_fd( terms[count]) ;
+			ptyfd = ml_term_get_master_fd( terms[count]) ;
 			FD_SET( ptyfd , &read_fds) ;
 
 			if( ptyfd > maxfd)
@@ -1621,7 +1561,7 @@ receive_next_event(void)
 		ml_term_flush( terms[count]) ;
 	#endif
 	
-		if( FD_ISSET( ml_term_get_pty_fd( terms[count]) , &read_fds))
+		if( FD_ISSET( ml_term_get_master_fd( terms[count]) , &read_fds))
 		{
 			ml_term_parse_vt100_sequence( terms[count]) ;
 		}
@@ -1665,8 +1605,6 @@ x_term_manager_init(
 	kik_conf_t *  conf ;
 	char *  value ;
 	int  use_xim ;
-	u_int  min_font_size ;
-	u_int  max_font_size ;
 	char *  invalid_msg = "%s %s is not valid.\n" ;
 
 	if( ! x_color_config_init( &color_config))
@@ -1697,7 +1635,9 @@ x_term_manager_init(
 	}
 
 
-	if( ( conf = conf_new()) == NULL)
+	kik_init_prog( argv[0] , DETAIL_VERSION) ;
+
+	if( ( conf = kik_conf_new()) == NULL)
 	{
 		return  0 ;
 	}
@@ -1705,12 +1645,13 @@ x_term_manager_init(
 	x_prepare_for_main_config( conf) ;
 
 	/*
+	 * Same processing as vte_terminal_class_init().
 	 * Following options are not possible to specify as arguments of mlclient.
 	 * 1) Options which are used only when mlterm starts up and which aren't
 	 *    changed dynamically. (e.g. "startup_screens")
 	 * 2) Options which change status of all ptys or windows. (Including ones
 	 *    which are possible to change dynamically.)
-	 *    (e.g. "button3_behavior")
+	 *    (e.g. "font_size_range")
 	 */
 
 	kik_conf_add_opt( conf , '@' , "screens" , 0 , "startup_screens" ,
@@ -1761,9 +1702,11 @@ x_term_manager_init(
 		) ;
 	kik_conf_add_opt( conf , '\0' , "clip" , 1 , "use_clipboard" ,
 		"use CLIPBOARD (not only PRIMARY) selection [false]") ;
+	kik_conf_add_opt( conf , '\0' , "restart" , 1 , "auto_restart" ,
+		"restart mlterm automatically if an error like segv happens. [true]") ;
 #ifdef  USE_IM_CURSOR_COLOR
 	kik_conf_add_opt( conf , '\0' , "imcolor" , 0 , "im_cursor_color" ,
-		"cursor color when input method is activated.") ;
+		"cursor color when input method is activated. [false]") ;
 #endif
 
 	if( ! kik_conf_parse_args( conf , &argc , &argv))
@@ -1775,23 +1718,18 @@ x_term_manager_init(
 
 	if( ( value = kik_conf_get_value( conf , "font_size_range")))
 	{
-		if( ! get_font_size_range( &min_font_size , &max_font_size , value))
+		u_int  min_font_size ;
+		u_int  max_font_size ;
+
+		if( get_font_size_range( &min_font_size , &max_font_size , value))
+		{
+			x_set_font_size_range( min_font_size , max_font_size) ;
+		}
+		else
 		{
 			kik_msg_printf( invalid_msg , "font_size_range" , value) ;
-
-			/* default values are used */
-			min_font_size = 6 ;
-			max_font_size = 30 ;
 		}
 	}
-	else
-	{
-		/* default values are used */
-		min_font_size = 6 ;
-		max_font_size = 30 ;
-	}
-
-	x_set_font_size_range( min_font_size , max_font_size) ;
 
 	x_main_config_init( &main_config , conf , argc , argv) ;
 
@@ -1982,6 +1920,12 @@ x_term_manager_init(
 		}
 	}
 
+	if( ! ( value = kik_conf_get_value( conf , "auto_restart")) ||
+	    strcmp( value , "true") == 0)
+	{
+		ml_set_auto_restart_cmd( kik_get_prog_path()) ;
+	}
+
 #ifdef  USE_IM_CURSOR_COLOR
 	if( ( value = kik_conf_get_value( conf , "im_cursor_color")))
 	{
@@ -1992,14 +1936,6 @@ x_term_manager_init(
 	}
 #endif
 
-
-	if( ( version = kik_conf_get_version( conf)) == NULL)
-	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " version string is NULL\n") ;
-	#endif
-	}
-
 	kik_conf_delete( conf) ;
 
 
@@ -2059,10 +1995,8 @@ x_term_manager_init(
 int
 x_term_manager_final(void)
 {
-	int  count ;
+	u_int  count ;
 
-	free( version) ;
-	
 	x_main_config_final( &main_config) ;
 	
 	ml_free_word_separators() ;
