@@ -150,7 +150,8 @@ error2:
 
 error1:
 	win->is_transparent = 0 ;
-	win->pic_mod = NULL ;
+
+	/* win->pic_mod = NULL is done in set_transparent. */
 
 	return  0 ;
 }
@@ -184,7 +185,7 @@ set_transparent(
 	)
 {
 	Window  parent ;
-
+	
 	if( ! IS_INHERIT_TRANSPARENT(win))
 	{
 		/*
@@ -193,19 +194,16 @@ set_transparent(
 		 * ParentRelative mode of parent windows should be unset.
 		 */
 
-		if( x_root_pixmap_available( win->disp->display))
+		/* win->is_transparent is set appropriately in update_transparent_picture(). */
+		if( update_transparent_picture( win))
 		{
-			/*
-			 * win->is_transparent is set appropriately in
-			 * update_transparent_picture().
-			 */
-			return  update_transparent_picture( win) ;
+			return  1 ;
 		}
 		else
 		{
 			kik_msg_printf( "_X_ROOTPMAP_ID is not found."
 				" Trying ParentRelative for transparency instead.\n") ;
-			
+
 			if( ! x_picture_modifier_is_normal( win->pic_mod))
 			{
 				kik_msg_printf( "(brightness, contrast, gamma and alpha "
@@ -251,16 +249,30 @@ set_transparent(
 		Window  root ;
 		Window *  list ;
 		u_int  n ;
+		XWindowAttributes  attr ;
 
-		XQueryTree( win->disp->display , parent , &root , &parent , &list , &n) ;
-		XFree( list) ;
-
-		if( parent == DefaultRootWindow( win->disp->display))
+		if( ! XQueryTree( win->disp->display , parent , &root , &parent , &list , &n))
 		{
 			break ;
 		}
 
-		XSetWindowBackgroundPixmap( win->disp->display , parent , ParentRelative) ;
+		XFree( list) ;
+
+		if( ! parent || parent == root)
+		{
+			break ;
+		}
+
+		if( XGetWindowAttributes( win->disp->display , parent , &attr) &&
+		    attr.depth == win->disp->depth)
+		{
+			XSetWindowBackgroundPixmap( win->disp->display , parent ,
+				ParentRelative) ;
+		}
+		else
+		{
+			break ;
+		}
 	}
 
 	return  1 ;
@@ -332,13 +344,17 @@ notify_configure_to_children(
 		#endif
 			set_transparent( win) ;
 		}
+		else if( win->window_exposed)
+		{
+			(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+			x_window_clear_margin_area( win) ;
+		}
+	#if  0
 		else
 		{
-		#if  0
 			x_window_clear_all( win) ;
-		#endif
-			(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
 		}
+	#endif
 	}
 
 	for( count = 0 ; count < win->num_of_children ; count ++)
@@ -788,8 +804,12 @@ x_window_init(
 
 	win->gc = NULL ;
 
-	memset( &win->fg_color , 0 , sizeof(win->fg_color)) ;
-	memset( &win->bg_color , 0 , sizeof(win->bg_color)) ;
+	win->fg_color.pixel = 0xff000000 ;
+	win->fg_color.red = 0x0 ;
+	win->fg_color.green = 0x0 ;
+	win->fg_color.blue = 0x0 ;
+	win->fg_color.alpha = 0xff ;
+	memset( &win->bg_color , 0xff , sizeof(win->bg_color)) ;
 
 	win->parent_window = None ;
 	win->parent = NULL ;
@@ -803,9 +823,9 @@ x_window_init(
 
 	win->cursor_shape = 0 ;
 
-	win->event_mask = ExposureMask | VisibilityChangeMask | FocusChangeMask | PropertyChangeMask ;
+	win->event_mask = ExposureMask | FocusChangeMask | PropertyChangeMask ;
 
-	/* if visibility is partially obscured , scrollable will be 0. */
+	/* If wall picture is set, scrollable will be 0. */
 	win->is_scrollable = 1 ;
 
 	/*
@@ -890,19 +910,27 @@ x_window_final(
 	}
 
 	free( win->children) ;
-	
-	x_display_clear_selection( win->disp , win) ;
-	
-	x_xic_deactivate( win) ;
 
-	x_window_set_type_engine( win , TYPE_XCORE) ;
-
-	if( win->create_gc)
+	if( win->my_window)
 	{
-		x_gc_delete( win->gc) ;
-	}
+		x_display_clear_selection( win->disp , win) ;
 
-	XDestroyWindow( win->disp->display , win->my_window) ;
+		x_xic_deactivate( win) ;
+
+		/* Delete cairo/xft. */
+		x_window_set_type_engine( win , TYPE_XCORE) ;
+
+		XDestroyWindow( win->disp->display , win->my_window) ;
+
+		if( win->create_gc)
+		{
+			x_gc_delete( win->gc) ;
+		}
+	}
+	else
+	{
+		/* x_window_show() is not called yet. */
+	}
 
 	if( win->window_finalized)
 	{
@@ -912,6 +940,7 @@ x_window_final(
 	return  1 ;
 }
 
+
 /*
  * Call this function in window_realized event at first.
  */
@@ -1037,24 +1066,21 @@ x_window_set_wall_picture(
 		return  0 ;
 	}
 
-	if( win->event_mask & VisibilityChangeMask)
-	{
-		/* rejecting VisibilityNotify event. is_scrollable is always false */
-		win->event_mask &= ~VisibilityChangeMask ;
-		XSelectInput( win->disp->display , win->my_window , win->event_mask) ;
-		win->is_scrollable = 0 ;
-	}
-
 	XSetWindowBackgroundPixmap( win->disp->display , win->my_window , pic) ;
 	win->wall_picture_is_set = 1 ;
+	win->is_scrollable = 0 ;
 
 	if( win->window_exposed)
 	{
-	#if  0
-		x_window_clear_all( win) ;
-	#endif
 		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+		x_window_clear_margin_area( win) ;
 	}
+#if  0
+	else
+	{
+		x_window_clear_all( win) ;
+	}
+#endif
 
 	return  1 ;
 }
@@ -1092,28 +1118,23 @@ x_window_unset_wall_picture(
 		return  1 ;
 	}
 
-	if( !( win->event_mask & VisibilityChangeMask))
-	{
-		/* accepting VisibilityNotify event. is_scrollable is changed dynamically. */
-		win->event_mask |= VisibilityChangeMask ;
-		XSelectInput( win->disp->display , win->my_window , win->event_mask) ;
-
-		/* setting 0 in case the current status is VisibilityPartiallyObscured. */
-		win->is_scrollable = 0 ;
-	}
-
 	XSetWindowBackgroundPixmap( win->disp->display , win->my_window , None) ;
 	XSetWindowBackground( win->disp->display , win->my_window , win->bg_color.pixel) ;
 
 	win->wall_picture_is_set = 0 ;
+	win->is_scrollable = 1 ;
 
 	if( win->window_exposed)
 	{
-	#if  0
-		x_window_clear_all( win) ;
-	#endif
 		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+		x_window_clear_margin_area( win) ;
 	}
+#if  0
+	else
+	{
+		x_window_clear_all( win) ;
+	}
+#endif
 
 	return  1 ;
 }
@@ -1176,11 +1197,15 @@ x_window_unset_transparent(
 
 		if( win->window_exposed)
 		{
-		#if  0
-			x_window_clear_all( win) ;
-		#endif
 			(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
+			x_window_clear_margin_area( win) ;
 		}
+	#if  0
+		else
+		{
+			x_window_clear_all( win) ;
+		}
+	#endif
 	}
 
 	for( count = 0 ; count < win->num_of_children ; count ++)
@@ -1791,8 +1816,39 @@ x_window_clear(
 	u_int	height
 	)
 {
-	XClearArea( win->disp->display , win->my_window ,
-		x + win->margin , y + win->margin , width , height , False) ;
+	if( x + width >= win->width)
+	{
+		/* Clearing margin area */
+		width += win->margin ;
+	}
+
+	if( x > 0)
+	{
+		x += win->margin ;
+	}
+	else
+	{
+		/* Clearing margin area */
+		width += win->margin ;
+	}
+
+	if( y + height >= win->height)
+	{
+		/* Clearing margin area */
+		height += win->margin ;
+	}
+
+	if( y > 0)
+	{
+		y += win->margin ;
+	}
+	else
+	{
+		/* Clearing margin area */
+		height += win->margin ;
+	}
+
+	XClearArea( win->disp->display , win->my_window , x , y , width , height , False) ;
 
 	return  1 ;
 }
@@ -1824,10 +1880,7 @@ x_window_clear_all(
 	x_window_t *  win
 	)
 {
-	XClearArea( win->disp->display , win->my_window , 0 , 0 ,
-		ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) , 0) ;
-
-	return  1 ;
+	return  x_window_clear( win , 0 , 0 , win->width , win->height) ;
 }
 
 int
@@ -2164,15 +2217,21 @@ x_window_receive_event(
 				RevertToParent , CurrentTime) ;
 		}
 	}
-	else if( event->type == Expose /* && event->xexpose.count == 0 */)
+	else if( event->type == Expose ||
+	         ( event->type == GraphicsExpose &&
+		   event->xgraphicsexpose.drawable == win->my_window) )
 	{
 		int  x ;
 		int  y ;
 		u_int  width ;
 		u_int  height ;
+		int  margin_area_exposed ;
 
+		margin_area_exposed = 0 ;
+		
 		if( event->xexpose.x < win->margin)
 		{
+			margin_area_exposed = 1 ;
 			x = 0 ;
 
 			if( x + event->xexpose.width > win->width)
@@ -2194,6 +2253,7 @@ x_window_receive_event(
 
 			if( x + event->xexpose.width > win->width)
 			{
+				margin_area_exposed = 1 ;
 				width = win->width - x ;
 			}
 			else
@@ -2204,6 +2264,7 @@ x_window_receive_event(
 
 		if( event->xexpose.y < win->margin)
 		{
+			margin_area_exposed = 1 ;
 			y = 0 ;
 
 			if( y + event->xexpose.height > win->height)
@@ -2225,6 +2286,7 @@ x_window_receive_event(
 
 			if( y + event->xexpose.height > win->height)
 			{
+				margin_area_exposed = 1 ;
 				height = win->height - y ;
 			}
 			else
@@ -2236,7 +2298,18 @@ x_window_receive_event(
 		if( win->window_exposed)
 		{
 			(*win->window_exposed)( win , x , y , width , height) ;
+
+			if( margin_area_exposed)
+			{
+				x_window_clear_margin_area( win) ;
+			}
 		}
+	#if  0
+		else
+		{
+			x_window_clear_all( win) ;
+		}
+	#endif
 	}
 	else if( event->type == ConfigureNotify)
 	{
@@ -2287,10 +2360,6 @@ x_window_receive_event(
 			win->width = event->xconfigure.width - win->margin * 2 ;
 			win->height = event->xconfigure.height - win->margin * 2 ;
 
-		#if  0
-			x_window_clear_all( win) ;
-		#endif
-
 			if( win->window_resized)
 			{
 				(*win->window_resized)( win) ;
@@ -2527,26 +2596,6 @@ x_window_receive_event(
 		XDeleteProperty( win->disp->display, event->xselection.requestor,
 			event->xselection.property) ;
 	}
-	else if( event->type == VisibilityNotify)
-	{
-		/* This event is changeable in run time. */
-
-		/*
-		 * VisibilityChangeMask is set or unset in run time(related to wall picture),
-		 * so check current VisibilityChangeMask state.
-		 */
-		if( win->event_mask & VisibilityChangeMask)
-		{
-			if( event->xvisibility.state == VisibilityPartiallyObscured)
-			{
-				win->is_scrollable = 0 ;
-			}
-			else if( event->xvisibility.state == VisibilityUnobscured)
-			{
-				win->is_scrollable = 1 ;
-			}
-		}
-	}
 	else if( event->type == ClientMessage)
 	{
 		if( event->xclient.format == 32 &&
@@ -2932,6 +2981,24 @@ x_window_draw_string(
 	u_int  len
 	)
 {
+	/* Removing trailing spaces. */
+	while( 1)
+	{
+		if( len == 0)
+		{
+			return  1 ;
+		}
+
+		if( *(str + len - 1) == ' ')
+		{
+			len-- ;
+		}
+		else
+		{
+			break ;
+		}
+	}
+
 	x_gc_set_fid( win->gc, font->xfont->fid) ;
 	x_gc_set_fg_color( win->gc, fg_color->pixel) ;
 
@@ -3455,8 +3522,8 @@ x_window_reset_group(
 int
 x_window_get_visible_geometry(
 	x_window_t *  win ,
-	int *  x ,		/* x relative to parent window */
-	int *  y ,		/* y relative to parent window */
+	int *  x ,		/* x relative to root window */
+	int *  y ,		/* y relative to root window */
 	int *  my_x ,		/* x relative to my window */
 	int *  my_y ,		/* y relative to my window */
 	u_int *  width ,
