@@ -10,6 +10,7 @@
 #include  <kiklib/kik_mem.h>	/* realloc/free */
 #include  <kiklib/kik_util.h>	/* K_MIN/K_MAX */
 #include  <mkf/mkf_codepoint_parser.h>
+#include  <ml_char.h>		/* UTF_MAX_SIZE */
 
 #include  "x_xic.h"
 #include  "x_picture.h"
@@ -41,6 +42,9 @@
 	GetSystemMetrics(SM_CYCAPTION)
 #endif
 
+#if  0
+#define  DEBUG_SCROLLABLE
+#endif
 
 #if  0
 #define  __DEBUG
@@ -60,7 +64,7 @@ static LONG  decorate_height ;		/* Height of Title bar etc. */
 static int
 set_transparent(
 	x_window_t *  win ,
-	x_picture_modifier_t *  pic_mod
+	int  alpha
 	)
 {
 /*
@@ -69,34 +73,19 @@ set_transparent(
  * in older winuser.h and libuser32.a(e.g. MSYS-DTK 1.0.1).
  */
 #if  defined(WS_EX_LAYERED) && defined(LWA_ALPHA)
-	int  count ;
-
-	if( ! win->parent)
+	if( ( win = x_get_root_window( win))->my_window)
 	{
-		/* Root Window */
-		if( win->my_window)
-		{
-			LONG  style ;
+		LONG  style ;
 
-			style = GetWindowLong( win->my_window , GWL_EXSTYLE) ;
-			SetWindowLong( win->my_window , GWL_EXSTYLE , style | WS_EX_LAYERED) ;
+		style = GetWindowLong( win->my_window , GWL_EXSTYLE) ;
+		SetWindowLong( win->my_window , GWL_EXSTYLE , style | WS_EX_LAYERED) ;
 
-		#if  1
-			SetLayeredWindowAttributes( win->my_window , 0 , pic_mod->alpha ,
-				LWA_ALPHA) ;
-		#else
-			SetLayeredWindowAttributes( win->my_window , win->bg_color.pixel ,
-				0 , LWA_COLORKEY) ;
-		#endif
-		}
-	}
-
-	win->is_transparent = 1 ;
-	win->pic_mod = pic_mod ;
-
-	for( count = 0 ; count < win->num_of_children ; count ++)
-	{
-		set_transparent( win->children[count] , win->pic_mod) ;
+	#if  1
+		SetLayeredWindowAttributes( win->my_window , 0 , alpha , LWA_ALPHA) ;
+	#else
+		SetLayeredWindowAttributes( win->my_window ,
+			ARGB_TO_RGB(win->bg_color.pixel) , 0 , LWA_COLORKEY) ;
+	#endif
 	}
 
 	return  1 ;
@@ -116,27 +105,12 @@ unset_transparent(
  * in older winuser.h and libuser32.a(e.g. MSYS-DTK 1.0.1).
  */
 #if  defined(WS_EX_LAYERED) && defined(LWA_ALPHA)
-	int  count ;
-	
-	if( ! win->parent)
+	if( ( win = x_get_root_window( win))->my_window)
 	{
-		/* Root Window */
-		
-		if( win->my_window)
-		{
-			LONG  style ;
+		LONG  style ;
 
-			style = GetWindowLong( win->my_window , GWL_EXSTYLE) ;
-			SetWindowLong( win->my_window , GWL_EXSTYLE , style & ~WS_EX_LAYERED) ;
-		}
-	}
-	
-	win->is_transparent = 0 ;
-	win->pic_mod = NULL ;
-
-	for( count = 0 ; count < win->num_of_children ; count ++)
-	{
-		unset_transparent( win->children[count]) ;
+		style = GetWindowLong( win->my_window , GWL_EXSTYLE) ;
+		SetWindowLong( win->my_window , GWL_EXSTYLE , style & ~WS_EX_LAYERED) ;
 	}
 
 	return  1 ;
@@ -145,12 +119,12 @@ unset_transparent(
 #endif
 }
 
+#ifndef  USE_WIN32GUI
 static int
 update_pic_transparent(
 	x_window_t *  win
 	)
 {
-#ifndef  USE_WIN32GUI
 	x_bg_picture_t  pic ;
 
 	if( ! x_bg_picture_init( &pic , win , win->pic_mod))
@@ -168,10 +142,10 @@ update_pic_transparent(
 	set_transparent( win , pic.pixmap) ;
 
 	x_bg_picture_final( &pic) ;
-#endif
 
 	return  1 ;
 }
+#endif
 
 /*
  * XXX
@@ -184,7 +158,7 @@ check_scrollable(
 {
 	if( win->is_focused)
 	{
-	#if  0
+	#ifdef  DEBUG_SCROLLABLE
 		kik_debug_printf( "SCREEN W %d H %d WINDOW W %d H %d X %d Y %d " ,
 			GetSystemMetrics( SM_CXSCREEN) , GetSystemMetrics( SM_CYSCREEN) ,
 			ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) , win->x , win->y) ;
@@ -196,7 +170,7 @@ check_scrollable(
 
 		if( win->y < 0 || win->x < 0)
 		{
-		#if  0
+		#ifdef  DEBUG_SCROLLABLE
 			kik_debug_printf( "NOT SCROLLABLE(1)\n") ;
 		#endif
 			return  0 ;
@@ -209,7 +183,7 @@ check_scrollable(
 
 			if( win->y + ACTUAL_HEIGHT(win) > screen_height)
 			{
-			#if  0
+			#ifdef  DEBUG_SCROLLABLE
 				kik_debug_printf( "NOT SCROLLABLE(2)\n") ;
 			#endif
 				return  0 ;
@@ -222,7 +196,7 @@ check_scrollable(
 
 				if( win->x + ACTUAL_WIDTH(win) > screen_width)
 				{
-				#if  0
+				#ifdef  DEBUG_SCROLLABLE
 					kik_debug_printf( "NOT SCROLLABLE(3)\n") ;
 				#endif
 					return  0 ;
@@ -237,7 +211,7 @@ check_scrollable(
 
 					SHAppBarMessage( ABM_GETTASKBARPOS , &barinfo) ;
 
-				#if  0
+				#ifdef  DEBUG_SCROLLABLE
 					kik_debug_printf( "TASKBAR t %d b %d l %d r %d " ,
 						barinfo.rc.top , barinfo.rc.bottom ,
 						barinfo.rc.left , barinfo.rc.right) ;
@@ -252,7 +226,7 @@ check_scrollable(
 								/* North */
 								if( win->y < barinfo.rc.bottom)
 								{
-								#if  0
+								#ifdef  DEBUG_SCROLLABLE
 									kik_debug_printf(
 									"NOT SCROLLABLE(4)\n") ;
 								#endif
@@ -264,7 +238,7 @@ check_scrollable(
 								/* West */
 								if( win->x < barinfo.rc.right)
 								{
-								#if  0
+								#ifdef  DEBUG_SCROLLABLE
 									kik_debug_printf(
 									"NOT SCROLLABLE(5)\n") ;
 								#endif
@@ -278,7 +252,7 @@ check_scrollable(
 							if( win->x + ACTUAL_WIDTH(win)
 								> barinfo.rc.left)
 							{
-							#if  0
+							#ifdef  DEBUG_SCROLLABLE
 								kik_debug_printf(
 								"NOT SCROLLABLE(6)\n") ;
 							#endif
@@ -291,14 +265,14 @@ check_scrollable(
 						/* South */
 						if( win->y + ACTUAL_HEIGHT(win) > barinfo.rc.top)
 						{
-						#if  0
+						#ifdef  DEBUG_SCROLLABLE
 							kik_debug_printf( "NOT SCROLLABLE(7)\n") ;
 						#endif
 							return  0 ;
 						}
 					}
 				
-				#if  0
+				#ifdef  DEBUG_SCROLLABLE
 					kik_debug_printf( "SCROLLABLE\n") ;
 				#endif
 					return  1 ;
@@ -308,7 +282,7 @@ check_scrollable(
 	}
 	else
 	{
-	#if  0
+	#ifdef  DEBUG_SCROLLABLE
 		kik_debug_printf( "NOT SCROLLABLE(4)\n") ;
 	#endif
 		return  0 ;
@@ -329,8 +303,8 @@ notify_focus_in_to_children(
 
 	win->is_focused = 1 ;
 
-	/* If win->wall_picture_is_set is true, is_scrollable is always 0. */
-	if( ! win->wall_picture_is_set)
+	/* If win->wall_picture is set, is_scrollable is always 0. */
+	if( ! win->wall_picture)
 	{
 		if( ! win->parent)
 		{
@@ -418,73 +392,6 @@ notify_move_to_children(
 	}
 }
 
-static void
-notify_configure_to_children(
-	x_window_t *  win
-	)
-{
-	int  count ;
-
-	if( win->is_transparent)
-	{
-		if( win->pic_mod || x_root_pixmap_available( win->disp->display))
-		{
-			update_pic_transparent( win) ;
-		}
-		else
-		{
-			x_window_clear_all( win) ;
-			(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
-		}
-	}
-
-	for( count = 0 ; count < win->num_of_children ; count ++)
-	{
-		notify_configure_to_children( win->children[count]) ;
-	}
-}
-
-static void
-notify_reparent_to_children(
-	x_window_t *  win
-	)
-{
-	int  count ;
-
-	/*
-	 * XXX
-	 * x_window_set_transparent() must be called once in win32,
-	 * so following code should be modified.
-	 */
-	if( win->is_transparent)
-	{
-		x_window_set_transparent( win , win->pic_mod) ;
-	}
-
-	for( count = 0 ; count < win->num_of_children ; count ++)
-	{
-		notify_reparent_to_children( win->children[count]) ;
-	}
-}
-
-static void
-notify_property_to_children(
-	x_window_t *  win
-	)
-{
-	int  count ;
-
-	if( win->is_transparent && x_root_pixmap_available( win->disp->display))
-	{
-		update_pic_transparent( win) ;
-	}
-
-	for( count = 0 ; count < win->num_of_children ; count ++)
-	{
-		notify_property_to_children( win->children[count]) ;
-	}
-}
-
 #if  0
 /*
  * Not used for now in win32.
@@ -730,7 +637,7 @@ draw_string(
 
 	if( font->conv)
 	{
-		if( ( str2 = malloc( len * 8)))	/* assume utf8 */
+		if( ( str2 = alloca( len * UTF_MAX_SIZE)))	/* assume utf8 */
 		{
 			(*m_cp_parser->init)( m_cp_parser) ;
 			/* 3rd argument of cp_parser->set_str is len(16bit) + cs(16bit) */
@@ -738,9 +645,8 @@ draw_string(
 							len | (FONT_CS(font->id) << 16) ) ;
 			
 			(*font->conv->init)( font->conv) ;
-			len = (*font->conv->convert)( font->conv, str2, len * 8, m_cp_parser) ;
-
-			if( len > 0)
+			if( ( len = (*font->conv->convert)( font->conv, str2, len * UTF_MAX_SIZE,
+					m_cp_parser)) > 0)
 			{
 				str = str2 ;
 			}
@@ -770,11 +676,6 @@ draw_string(
 		SetBkMode( win->gc->gc , OPAQUE) ;
 	}
 
-	if( str2)
-	{
-		free( str2) ;
-	}
-
 	return  1 ;
 }
 
@@ -845,8 +746,8 @@ x_window_init(
 	win->parent_window = None ;
 	win->my_window = None ;
 
-	memset( &win->fg_color , 0 , sizeof(win->fg_color)) ;
-	memset( &win->bg_color , 0 , sizeof(win->bg_color)) ;
+	win->fg_color.pixel = 0xff000000 ;
+	win->bg_color.pixel = 0xffffffff ;
 
 	win->parent = NULL ;
 	win->children = NULL ;
@@ -854,9 +755,8 @@ x_window_init(
 
 	win->pic_mod = NULL ;
 
-	win->wall_picture_is_set = 0 ;
-	win->is_transparent = 0 ;
 	win->wall_picture = None ;
+	win->is_transparent = 0 ;
 
 	win->cursor_shape = 0 ;
 
@@ -1027,18 +927,13 @@ x_window_set_wall_picture(
 	Pixmap  pic
 	)
 {
-	if( win->is_transparent)
-	{
-		/*
-		 * unset transparent before setting wall picture !
-		 */
-
-		return  0 ;
-	}
-
 	win->is_scrollable = 0 ;
 	win->wall_picture = pic ;
-	win->wall_picture_is_set = 1 ;
+
+	if( win->my_window != None)
+	{
+		InvalidateRect( win->my_window, NULL, FALSE) ;
+	}
 
 	return  1 ;
 }
@@ -1048,30 +943,13 @@ x_window_unset_wall_picture(
 	x_window_t *  win
 	)
 {
-#if  0
-	if( ! win->wall_picture_is_set)
-	{
-		/* already unset */
-
-		return  1 ;
-	}
-#endif
-
-	if( win->is_transparent)
-	{
-		/*
-		 * transparent background is not a wall picture :)
-		 * this case is regarded as not using a wall picture.
-		 */
-
-		return  1 ;
-	}
-
-#if  0
-	win->is_scrollable = 1 ;
-#endif
+	win->is_scrollable = check_scrollable( win) ;
 	win->wall_picture = None ;
-	win->wall_picture_is_set = 0 ;
+
+	if( win->my_window != None)
+	{
+		InvalidateRect( win->my_window, NULL, FALSE) ;
+	}
 
 	return  1 ;
 }
@@ -1082,12 +960,7 @@ x_window_set_transparent(
 	x_picture_modifier_t *  pic_mod
 	)
 {
-	if( ! pic_mod)
-	{
-		return  0 ;
-	}
-	
-	return  set_transparent( x_get_root_window( win) , pic_mod) ;
+	return  0 ;
 }
 
 int
@@ -1095,7 +968,7 @@ x_window_unset_transparent(
 	x_window_t *  win
 	)
 {
-	return  unset_transparent( x_get_root_window( win)) ;
+	return  0 ;
 }
 
 int
@@ -1131,11 +1004,27 @@ x_window_set_bg_color(
 	x_color_t *  bg_color
 	)
 {
+	int  alpha ;
+
 	if( win->bg_color.pixel == bg_color->pixel)
 	{
 		return  0 ;
 	}
 
+	if( ( alpha = ((bg_color->pixel >> 24) & 0xff)) != 0xff)
+	{
+		set_transparent( win , alpha) ;
+	}
+	else if( ((win->bg_color.pixel >> 24) & 0xff) != 0xff)
+	{
+		/*
+		 * If alpha is changed from less than 0xff to 0xff,
+		 * transparent is disabled.
+		 * XXX It is assumed that alpha is changed by only one window.
+		 */
+		unset_transparent( win) ;
+	}
+
 	win->bg_color = *bg_color ;
 
 	if( win->my_window != None)
@@ -1497,12 +1386,44 @@ x_window_clear(
 {
 	RECT  r ;
 
-	r.left = x + win->margin ;
-	r.top = y + win->margin ;
+	if( x + width >= win->width)
+	{
+		/* Clearing margin area */
+		width += win->margin ;
+	}
+
+	if( x > 0)
+	{
+		x += win->margin ;
+	}
+	else
+	{
+		/* Clearing margin area */
+		width += win->margin ;
+	}
+
+	if( y + height >= win->height)
+	{
+		/* Clearing margin area */
+		height += win->margin ;
+	}
+
+	if( y > 0)
+	{
+		y += win->margin ;
+	}
+	else
+	{
+		/* Clearing margin area */
+		height += win->margin ;
+	}
+
+	r.left = x ;
+	r.top = y ;
 	
-	/* XXX Garbage is left in screen in scrolling without +1. Related to NULL_PEN ? */
-	r.right = x + win->margin + width + 1 ;
-	r.bottom = y + win->margin + height + 1 ;
+	/* XXX Garbage is left in screen in scrolling without +1 due to NULL_PEN ? */
+	r.right = x + width /* + 1 */ ;
+	r.bottom = y + height /* + 1 */ ;
 
 	if( win->gc->gc == None)
 	{
@@ -1592,7 +1513,7 @@ x_window_clear_all(
 	x_window_t *  win
 	)
 {
-	return  x_window_clear( win, 0, 0, win->width, win->height) ;
+	return  x_window_clear( win , 0 , 0 , win->width , win->height) ;
 }
 
 int
@@ -1817,6 +1738,7 @@ x_window_receive_event(
 		if( win->window_exposed)
 		{
 			PAINTSTRUCT  ps ;
+			int  margin_area_exposed ;
 			int  x ;
 			int  y ;
 			u_int  width ;
@@ -1824,8 +1746,11 @@ x_window_receive_event(
 
 			x_set_gc( win->gc, BeginPaint( win->my_window, &ps)) ;
 
+			margin_area_exposed = 0 ;
+
 			if( ps.rcPaint.left < win->margin)
 			{
+				margin_area_exposed = 1 ;
 				x = 0 ;
 			}
 			else
@@ -1835,6 +1760,7 @@ x_window_receive_event(
 
 			if( ps.rcPaint.top < win->margin)
 			{
+				margin_area_exposed = 1 ;
 				y = 0 ;
 			}
 			else
@@ -1844,6 +1770,7 @@ x_window_receive_event(
 
 			if( ps.rcPaint.right > win->width - win->margin)
 			{
+				margin_area_exposed = 1 ;
 				width = win->width - win->margin - x ;
 			}
 			else
@@ -1854,6 +1781,7 @@ x_window_receive_event(
 
 			if( ps.rcPaint.bottom > win->height - win->margin)
 			{
+				margin_area_exposed = 1 ;
 				height = win->height - win->margin - y ;
 			}
 			else
@@ -1864,6 +1792,11 @@ x_window_receive_event(
 
 			(*win->window_exposed)( win, x, y, width, height) ;
 
+			if( margin_area_exposed)
+			{
+				x_window_clear_margin_area( win) ;
+			}
+
 			EndPaint( win->my_window, &ps) ;
 			x_set_gc( win->gc, None) ;
 
@@ -2610,6 +2543,24 @@ x_window_draw_string(
 		return  0 ;
 	}
 
+	/* Removing trailing spaces. */
+	while( 1)
+	{
+		if( len == 0)
+		{
+			return  1 ;
+		}
+
+		if( *(str + len - 1) == ' ')
+		{
+			len-- ;
+		}
+		else
+		{
+			break ;
+		}
+	}
+
 	x_gc_set_fid( win->gc, font->fid) ;
 	x_gc_set_fg_color( win->gc, fg_color->pixel) ;
 	
